/*----- PROTECTED REGION ID(PZ116E.cpp) ENABLED START -----*/
static const char *RcsId = "$Id: PZ116E.cpp,v 1.9 2015-08-27 14:51:04 giulio Exp $";
//=============================================================================
//
// file :        PZ116E.cpp
//
// description : C++ source for the PZ116E and its commands.
//               The class is derived from Device. It represents the
//               CORBA servant object which will be accessed from the
//               network. All commands which can be executed on the
//               PZ116E are implemented in this file.
//
// project :     PI Piezo PZ116E controller device server.
//
// $Author: giulio $
//
// $Revision: 1.9 $
// $Date: 2015-08-27 14:51:04 $
//
// SVN only:
// $HeadURL:  $
//
// CVS only:
// $Source: /home/cvsadm/cvsroot/fermi/servers/pz116e/src/PZ116E.cpp,v $
// $Log: PZ116E.cpp,v $
// Revision 1.9  2015-08-27 14:51:04  giulio
// Supported open-loop mode
//
// Revision 1.8  2013-12-30 11:10:54  giulio
// Moved to Tango8
//
// Revision 1.7  2012-07-02 08:27:40  giulio
// Reset errors before rtmode/reset commands
//
// Revision 1.6  2012-06-28 08:19:30  giulio
// Modified rtmode attribute
//
// Revision 1.5  2012-06-11 15:35:07  giulio
// fixed waveform write attribute
//
// Revision 1.4  2012-06-11 15:11:54  giulio
// fixed send_cmd command inout
//
// Revision 1.3  2012-06-07 09:57:21  giulio
// Added status strings, load waveform in init procedure
//
// Revision 1.2  2012-06-06 14:35:08  giulio
// Fixed sendcmd bug
//
//
//=============================================================================
//                This file is generated by POGO
//        (Program Obviously used to Generate tango Object)
//=============================================================================


#include <PZ116E.h>
#include <PZ116EClass.h>

const char err_msg_table[PZ116E_NUM_ERRORS][PZ116E_ERR_MSG_SIZE] = {
	"0", "no errror",
	"1", "Parameter syntax error",
	"2", "Unknown command",
	"3", "Command length out of limits or command buffer overrun",
	"5", "Unallowable move attempted on unreferenced axis, or move attempted with servo off",
	"10", "Controller was stopped by command",
	"15", "Invalid axis identifier",
	"17", "Parameter out of range",
	"20", "Macro not found",
	"54", "Unknown parameter",
	"56", "Password invalid",
	"60", "Protected Param: current Command Level (CCL) too low",
	"73", "Motion commands are not allowed when wave generator is active",
	"79", "Open-loop commands (SVA, SVR) are not allowed when servo is on",
	"89", "Command not allowed in current motion mode",
	"210", "Illegal file name (must be 8-0 format)",
	"232", "Save system configuration failed",
	"233", "Load system configuration failed",
	"306", "Error on I2C bus",
	"309", "Insufficient space to store macro",
	"405", "Wave parameter out of range"
	};

/*----- PROTECTED REGION END -----*/	//	PZ116E.cpp

/**
 *  PZ116E class description:
 *    PI Piezo PZ116E controller device server
 */

//================================================================
//  The following table gives the correspondence
//  between command and method names.
//
//  Command name   |  Method name
//================================================================
//  State          |  Inherited (no method)
//  Status         |  Inherited (no method)
//  Forward        |  forward
//  Backward       |  backward
//  Stop           |  stop
//  Reset          |  reset
//  StartWaveform  |  start_waveform
//  StopWaveform   |  stop_waveform
//  SendCmd        |  send_cmd
//================================================================

//================================================================
//  Attributes managed are:
//================================================================
//  RtMode           |  Tango::DevBoolean	Scalar
//  Position         |  Tango::DevDouble	Scalar
//  IdentityNumber   |  Tango::DevString	Scalar
//  OneshotWaveform  |  Tango::DevBoolean	Scalar
//  Stat             |  Tango::DevBoolean	Spectrum  ( max = 32)
//  Waveform         |  Tango::DevDouble	Spectrum  ( max = 256)
//================================================================

namespace PZ116E_ns
{
/*----- PROTECTED REGION ID(PZ116E::namespace_starting) ENABLED START -----*/

	//	static initializations

	/*----- PROTECTED REGION END -----*/	//	PZ116E::namespace_starting

//--------------------------------------------------------
/**
 *	Method      : PZ116E::PZ116E()
 *	Description : Constructors for a Tango device
 *                implementing the classPZ116E
 */
//--------------------------------------------------------
PZ116E::PZ116E(Tango::DeviceClass *cl, string &s)
 : TANGO_BASE_CLASS(cl, s.c_str())
{
	/*----- PROTECTED REGION ID(PZ116E::constructor_1) ENABLED START -----*/

	init_device();

	/*----- PROTECTED REGION END -----*/	//	PZ116E::constructor_1
}
//--------------------------------------------------------
PZ116E::PZ116E(Tango::DeviceClass *cl, const char *s)
 : TANGO_BASE_CLASS(cl, s)
{
	/*----- PROTECTED REGION ID(PZ116E::constructor_2) ENABLED START -----*/

	init_device();

	/*----- PROTECTED REGION END -----*/	//	PZ116E::constructor_2
}
//--------------------------------------------------------
PZ116E::PZ116E(Tango::DeviceClass *cl, const char *s, const char *d)
 : TANGO_BASE_CLASS(cl, s, d)
{
	/*----- PROTECTED REGION ID(PZ116E::constructor_3) ENABLED START -----*/

	init_device();

	/*----- PROTECTED REGION END -----*/	//	PZ116E::constructor_3
}

//--------------------------------------------------------
/**
 *	Method      : PZ116E::delete_device()
 *	Description : will be called at device destruction or at init command
 */
//--------------------------------------------------------
void PZ116E::delete_device()
{
	DEBUG_STREAM << "PZ116E::delete_device() " << device_name << endl;
	/*----- PROTECTED REGION ID(PZ116E::delete_device) ENABLED START -----*/

	//	Delete device allocated objects

	/*----- PROTECTED REGION END -----*/	//	PZ116E::delete_device
	delete[] attr_RtMode_read;
	delete[] attr_Position_read;
	delete[] attr_IdentityNumber_read;
	delete[] attr_OneshotWaveform_read;
	delete[] attr_Stat_read;
	delete[] attr_Waveform_read;
}

//--------------------------------------------------------
/**
 *	Method      : PZ116E::init_device()
 *	Description : will be called at device initialization.
 */
//--------------------------------------------------------
void PZ116E::init_device()
{
	DEBUG_STREAM << "PZ116E::init_device() create device " << device_name << endl;
	/*----- PROTECTED REGION ID(PZ116E::init_device_before) ENABLED START -----*/

	//	Initialization before get_device_property() call

	/*----- PROTECTED REGION END -----*/	//	PZ116E::init_device_before
	

	//	Get the device properties from database
	get_device_property();
	
	attr_RtMode_read = new Tango::DevBoolean[1];
	attr_Position_read = new Tango::DevDouble[1];
	attr_IdentityNumber_read = new Tango::DevString[1];
	attr_OneshotWaveform_read = new Tango::DevBoolean[1];
	attr_Stat_read = new Tango::DevBoolean[32];
	attr_Waveform_read = new Tango::DevDouble[256];

	/*----- PROTECTED REGION ID(PZ116E::init_device) ENABLED START -----*/

	//	Initialize device
	mutex_cmd = new omni_mutex::omni_mutex();

	clock_gettime(CLOCK_MONOTONIC, &last_state_time);
	last_state_time.tv_sec -= 10;
	last_reset_time.tv_sec = last_state_time.tv_sec;
	last_pos_time.tv_sec = last_state_time.tv_sec;

	try {
		Tango::MultiAttribute *attr_list = get_device_attr();    
		attr_position = 0;
		attr_position = &(attr_list->get_w_attr_by_name("Position"));
	}
	catch(...) {
		INFO_STREAM << "PZ116E::init_device(): error getting position attribute" << endl;
	} 		

	for (int i = 0; i < PZ116E_NUM_STATES; i++) {
		attr_Stat_read[i] = false;
	}

	for (int i = 0; i < 256; i++)
		attr_Waveform_read[i] = 0;
	wsize = 0;
	start_wave_idx = 0;
	stop_wave_idx = 0;
	
	open_tty();

	try {
		check_err();
	}
	catch(Tango::DevFailed e) {}
	try {
		init_pi();
	}
	catch(Tango::DevFailed e) {
		set_state(Tango::FAULT); 
		set_status("Initialization failed\n");
	}

	*attr_RtMode_read = false;
	*attr_IdentityNumber_read = idn_name;
	

	/*----- PROTECTED REGION END -----*/	//	PZ116E::init_device
}

//--------------------------------------------------------
/**
 *	Method      : PZ116E::get_device_property()
 *	Description : Read database to initialize property data members.
 */
//--------------------------------------------------------
void PZ116E::get_device_property()
{
	/*----- PROTECTED REGION ID(PZ116E::get_device_property_before) ENABLED START -----*/

	//	Initialize property data members

	/*----- PROTECTED REGION END -----*/	//	PZ116E::get_device_property_before


	//	Read device properties from database.
	Tango::DbData	dev_prop;
	dev_prop.push_back(Tango::DbDatum("InitCommand"));
	dev_prop.push_back(Tango::DbDatum("SerialLine"));
	dev_prop.push_back(Tango::DbDatum("Um2PsConvFactor"));
	dev_prop.push_back(Tango::DbDatum("TxRxSleep"));
	dev_prop.push_back(Tango::DbDatum("ClosedLoop"));

	//	is there at least one property to be read ?
	if (dev_prop.size()>0)
	{
		//	Call database and extract values
		if (Tango::Util::instance()->_UseDb==true)
			get_db_device()->get_property(dev_prop);
	
		//	get instance on PZ116EClass to get class property
		Tango::DbDatum	def_prop, cl_prop;
		PZ116EClass	*ds_class =
			(static_cast<PZ116EClass *>(get_device_class()));
		int	i = -1;

		//	Try to initialize InitCommand from class property
		cl_prop = ds_class->get_class_property(dev_prop[++i].name);
		if (cl_prop.is_empty()==false)	cl_prop  >>  initCommand;
		else {
			//	Try to initialize InitCommand from default device value
			def_prop = ds_class->get_default_device_property(dev_prop[i].name);
			if (def_prop.is_empty()==false)	def_prop  >>  initCommand;
		}
		//	And try to extract InitCommand value from database
		if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  initCommand;

		//	Try to initialize SerialLine from class property
		cl_prop = ds_class->get_class_property(dev_prop[++i].name);
		if (cl_prop.is_empty()==false)	cl_prop  >>  serialLine;
		else {
			//	Try to initialize SerialLine from default device value
			def_prop = ds_class->get_default_device_property(dev_prop[i].name);
			if (def_prop.is_empty()==false)	def_prop  >>  serialLine;
		}
		//	And try to extract SerialLine value from database
		if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  serialLine;

		//	Try to initialize Um2PsConvFactor from class property
		cl_prop = ds_class->get_class_property(dev_prop[++i].name);
		if (cl_prop.is_empty()==false)	cl_prop  >>  um2PsConvFactor;
		else {
			//	Try to initialize Um2PsConvFactor from default device value
			def_prop = ds_class->get_default_device_property(dev_prop[i].name);
			if (def_prop.is_empty()==false)	def_prop  >>  um2PsConvFactor;
		}
		//	And try to extract Um2PsConvFactor value from database
		if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  um2PsConvFactor;

		//	Try to initialize TxRxSleep from class property
		cl_prop = ds_class->get_class_property(dev_prop[++i].name);
		if (cl_prop.is_empty()==false)	cl_prop  >>  txRxSleep;
		else {
			//	Try to initialize TxRxSleep from default device value
			def_prop = ds_class->get_default_device_property(dev_prop[i].name);
			if (def_prop.is_empty()==false)	def_prop  >>  txRxSleep;
		}
		//	And try to extract TxRxSleep value from database
		if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  txRxSleep;

		//	Try to initialize ClosedLoop from class property
		cl_prop = ds_class->get_class_property(dev_prop[++i].name);
		if (cl_prop.is_empty()==false)	cl_prop  >>  closedLoop;
		else {
			//	Try to initialize ClosedLoop from default device value
			def_prop = ds_class->get_default_device_property(dev_prop[i].name);
			if (def_prop.is_empty()==false)	def_prop  >>  closedLoop;
		}
		//	And try to extract ClosedLoop value from database
		if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  closedLoop;

	}

	/*----- PROTECTED REGION ID(PZ116E::get_device_property_after) ENABLED START -----*/

	//	Check device property data members init

	/*----- PROTECTED REGION END -----*/	//	PZ116E::get_device_property_after
}

//--------------------------------------------------------
/**
 *	Method      : PZ116E::always_executed_hook()
 *	Description : method always executed before any command is executed
 */
//--------------------------------------------------------
void PZ116E::always_executed_hook()
{
	INFO_STREAM << "PZ116E::always_executed_hook()  " << device_name << endl;
	/*----- PROTECTED REGION ID(PZ116E::always_executed_hook) ENABLED START -----*/

	//	code always executed before all requests
	set_DevState();

	/*----- PROTECTED REGION END -----*/	//	PZ116E::always_executed_hook
}

//--------------------------------------------------------
/**
 *	Method      : PZ116E::read_attr_hardware()
 *	Description : Hardware acquisition for attributes
 */
//--------------------------------------------------------
void PZ116E::read_attr_hardware(TANGO_UNUSED(vector<long> &attr_list))
{
	DEBUG_STREAM << "PZ116E::read_attr_hardware(vector<long> &attr_list) entering... " << endl;
	/*----- PROTECTED REGION ID(PZ116E::read_attr_hardware) ENABLED START -----*/

	//	Add your own code

	/*----- PROTECTED REGION END -----*/	//	PZ116E::read_attr_hardware
}
//--------------------------------------------------------
/**
 *	Method      : PZ116E::write_attr_hardware()
 *	Description : Hardware writing for attributes
 */
//--------------------------------------------------------
void PZ116E::write_attr_hardware(TANGO_UNUSED(vector<long> &attr_list))
{
	DEBUG_STREAM << "PZ116E::write_attr_hardware(vector<long> &attr_list) entering... " << endl;
	/*----- PROTECTED REGION ID(PZ116E::write_attr_hardware) ENABLED START -----*/
	
	//	Add your own code
	
	/*----- PROTECTED REGION END -----*/	//	PZ116E::write_attr_hardware
}

//--------------------------------------------------------
/**
 *	Read attribute RtMode related method
 *	Description: If enabled, do not readback position, do not check errors in mov command
 *
 *	Data type:	Tango::DevBoolean
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void PZ116E::read_RtMode(Tango::Attribute &attr)
{
	DEBUG_STREAM << "PZ116E::read_RtMode(Tango::Attribute &attr) entering... " << endl;
	/*----- PROTECTED REGION ID(PZ116E::read_RtMode) ENABLED START -----*/

	//	Set the attribute value
	attr.set_value(attr_RtMode_read);

	/*----- PROTECTED REGION END -----*/	//	PZ116E::read_RtMode
}
//--------------------------------------------------------
/**
 *	Write attribute RtMode related method
 *	Description: If enabled, do not readback position, do not check errors in mov command
 *
 *	Data type:	Tango::DevBoolean
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void PZ116E::write_RtMode(Tango::WAttribute &attr)
{
	DEBUG_STREAM << "PZ116E::write_RtMode(Tango::WAttribute &attr) entering... " << endl;
	//	Retrieve write value
	Tango::DevBoolean	w_val;
	attr.get_write_value(w_val);
	/*----- PROTECTED REGION ID(PZ116E::write_RtMode) ENABLED START -----*/
	/* clean errors */
	try {
		check_err();
	}
	catch(Tango::DevFailed e) {}

	if (closedLoop) {
		if (w_val) {
			/* disable drift compensation */
			write_read((char *)"DCO A 0\r",NULL,false,0);
		}
		else {
			/* enable drift compensation */
			write_read((char *)"DCO A 1\r",NULL,false,0);
		}
		check_err();
	}

	*attr_RtMode_read = w_val;	

	/*----- PROTECTED REGION END -----*/	//	PZ116E::write_RtMode
}
//--------------------------------------------------------
/**
 *	Read attribute Position related method
 *	Description: 
 *
 *	Data type:	Tango::DevDouble
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void PZ116E::read_Position(Tango::Attribute &attr)
{
	DEBUG_STREAM << "PZ116E::read_Position(Tango::Attribute &attr) entering... " << endl;
	/*----- PROTECTED REGION ID(PZ116E::read_Position) ENABLED START -----*/

	//	Set the attribute value
	attr.set_value(attr_Position_read);

	/*----- PROTECTED REGION END -----*/	//	PZ116E::read_Position
}
//--------------------------------------------------------
/**
 *	Write attribute Position related method
 *	Description: 
 *
 *	Data type:	Tango::DevDouble
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void PZ116E::write_Position(Tango::WAttribute &attr)
{
	DEBUG_STREAM << "PZ116E::write_Position(Tango::WAttribute &attr) entering... " << endl;
	//	Retrieve write value
	Tango::DevDouble	w_val;
	attr.get_write_value(w_val);
	/*----- PROTECTED REGION ID(PZ116E::write_Position) ENABLED START -----*/
	char instr[PZ116E_STRING_SIZE];
	if (closedLoop) {
		sprintf(instr,"MOV A %.3f\r", (w_val / um2PsConvFactor));
	}
	else {
		sprintf(instr,"SVA A %.1f\r", (w_val / um2PsConvFactor));
	}
	write_read(instr,NULL,false,0);
	
	if (*attr_RtMode_read) {
		*attr_Position_read = w_val;
	}
	else {
		check_err();
	}

	/*----- PROTECTED REGION END -----*/	//	PZ116E::write_Position
}
//--------------------------------------------------------
/**
 *	Read attribute IdentityNumber related method
 *	Description: 
 *
 *	Data type:	Tango::DevString
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void PZ116E::read_IdentityNumber(Tango::Attribute &attr)
{
	DEBUG_STREAM << "PZ116E::read_IdentityNumber(Tango::Attribute &attr) entering... " << endl;
	/*----- PROTECTED REGION ID(PZ116E::read_IdentityNumber) ENABLED START -----*/

	//	Set the attribute value

	attr.set_value(attr_IdentityNumber_read);

	/*----- PROTECTED REGION END -----*/	//	PZ116E::read_IdentityNumber
}
//--------------------------------------------------------
/**
 *	Read attribute OneshotWaveform related method
 *	Description: 
 *
 *	Data type:	Tango::DevBoolean
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void PZ116E::read_OneshotWaveform(Tango::Attribute &attr)
{
	DEBUG_STREAM << "PZ116E::read_OneshotWaveform(Tango::Attribute &attr) entering... " << endl;
	/*----- PROTECTED REGION ID(PZ116E::read_OneshotWaveform) ENABLED START -----*/

	//	Set the attribute value
	attr.set_value(attr_OneshotWaveform_read);

	/*----- PROTECTED REGION END -----*/	//	PZ116E::read_OneshotWaveform
}
//--------------------------------------------------------
/**
 *	Write attribute OneshotWaveform related method
 *	Description: 
 *
 *	Data type:	Tango::DevBoolean
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void PZ116E::write_OneshotWaveform(Tango::WAttribute &attr)
{
	DEBUG_STREAM << "PZ116E::write_OneshotWaveform(Tango::WAttribute &attr) entering... " << endl;
	//	Retrieve write value
	Tango::DevBoolean	w_val;
	attr.get_write_value(w_val);
	/*----- PROTECTED REGION ID(PZ116E::write_OneshotWaveform) ENABLED START -----*/

	*	attr_OneshotWaveform_read = w_val;

	/*----- PROTECTED REGION END -----*/	//	PZ116E::write_OneshotWaveform
}
//--------------------------------------------------------
/**
 *	Read attribute Stat related method
 *	Description: 0) moving=true. not moving=false
 *               1) comm. error=true comm. ok=false
 *               
 *
 *	Data type:	Tango::DevBoolean
 *	Attr type:	Spectrum max = 32
 */
//--------------------------------------------------------
void PZ116E::read_Stat(Tango::Attribute &attr)
{
	DEBUG_STREAM << "PZ116E::read_Stat(Tango::Attribute &attr) entering... " << endl;
	/*----- PROTECTED REGION ID(PZ116E::read_Stat) ENABLED START -----*/

	//	Set the attribute value

	attr.set_value(attr_Stat_read, PZ116E_NUM_STATES);

	/*----- PROTECTED REGION END -----*/	//	PZ116E::read_Stat
}
//--------------------------------------------------------
/**
 *	Read attribute Waveform related method
 *	Description: 
 *
 *	Data type:	Tango::DevDouble
 *	Attr type:	Spectrum max = 256
 */
//--------------------------------------------------------
void PZ116E::read_Waveform(Tango::Attribute &attr)
{
	DEBUG_STREAM << "PZ116E::read_Waveform(Tango::Attribute &attr) entering... " << endl;
	/*----- PROTECTED REGION ID(PZ116E::read_Waveform) ENABLED START -----*/

	//	Set the attribute value
	attr.set_value(attr_Waveform_read,wsize);

	/*----- PROTECTED REGION END -----*/	//	PZ116E::read_Waveform
}
//--------------------------------------------------------
/**
 *	Write attribute Waveform related method
 *	Description: 
 *
 *	Data type:	Tango::DevDouble
 *	Attr type:	Spectrum max = 256
 */
//--------------------------------------------------------
void PZ116E::write_Waveform(Tango::WAttribute &attr)
{
	DEBUG_STREAM << "PZ116E::write_Waveform(Tango::WAttribute &attr) entering... " << endl;
	//	Retrieve number of write values
	int	w_length = attr.get_write_value_length();

	//	Retrieve pointer on write values (Do not delete !)
	const Tango::DevDouble	*w_val;
	attr.get_write_value(w_val);
	/*----- PROTECTED REGION ID(PZ116E::write_Waveform) ENABLED START -----*/
	/* clean errors */
	try {
		check_err();
	}
	catch(Tango::DevFailed e) {}


	/* load waveform */
	char instr[PZ116E_STRING_SIZE], outstr[PZ116E_STRING_SIZE];
	for (int i = 0; i < w_length; i++) {
		sprintf(instr,"SWT A %d %.3f\r", i, w_val[i]/um2PsConvFactor);
		write_read(instr, outstr, true, 10000);
		if (outstr[0] != '0') {
			TangoSys_MemStream out_stream;
  		out_stream << "Failed to write waveform position " << i << ", value=" << w_val[i] << ends;
			Tango::Except::throw_exception(
				out_stream.str(), (const char *) "Failed to write waveform",
				(const char *) "Z116E::write_Waveform()", Tango::ERR);
		}
	}
	if (*attr_OneshotWaveform_read) {
		for (int i = w_length; i < 256; i++) {
			sprintf(instr,"SWT A %d %.3f\r", i, w_val[w_length-1]/um2PsConvFactor);
			write_read(instr, outstr, true, 10000);
			if (outstr[0] != '0') {
				TangoSys_MemStream out_stream;
				out_stream << "Failed to write waveform position " << i << ", value=" << w_val[i] << ends;
				Tango::Except::throw_exception(
					out_stream.str(), (const char *) "Failed to write waveform",
					(const char *) "Z116E::write_Waveform()", Tango::ERR);
			}
		}
	}
	check_err();

	for (int i = 0; i < w_length; i++)
		attr_Waveform_read[i] = w_val[i];

	wsize = w_length;



	/*----- PROTECTED REGION END -----*/	//	PZ116E::write_Waveform
}

//--------------------------------------------------------
/**
 *	Method      : PZ116E::add_dynamic_attributes()
 *	Description : Create the dynamic attributes if any
 *                for specified device.
 */
//--------------------------------------------------------
void PZ116E::add_dynamic_attributes()
{
	/*----- PROTECTED REGION ID(PZ116E::add_dynamic_attributes) ENABLED START -----*/
	
	//	Add your own code to create and add dynamic attributes if any
	
	/*----- PROTECTED REGION END -----*/	//	PZ116E::add_dynamic_attributes
}

//--------------------------------------------------------
/**
 *	Command Forward related method
 *	Description: Move forward
 *
 *	@param argin 
 */
//--------------------------------------------------------
void PZ116E::forward(Tango::DevDouble argin)
{
	DEBUG_STREAM << "PZ116E::Forward()  - " << device_name << endl;
	/*----- PROTECTED REGION ID(PZ116E::forward) ENABLED START -----*/

	//	Add your own code
	char instr[PZ116E_STRING_SIZE];
	sprintf(instr,"MVR A %.3f\r", (argin / um2PsConvFactor));
	write_read(instr,NULL,false,0);
	/*----- PROTECTED REGION END -----*/	//	PZ116E::forward
}
//--------------------------------------------------------
/**
 *	Command Backward related method
 *	Description: Move backward
 *
 *	@param argin 
 */
//--------------------------------------------------------
void PZ116E::backward(Tango::DevDouble argin)
{
	DEBUG_STREAM << "PZ116E::Backward()  - " << device_name << endl;
	/*----- PROTECTED REGION ID(PZ116E::backward) ENABLED START -----*/

	//	Add your own code
	char instr[PZ116E_STRING_SIZE];
	sprintf(instr,"MVR A %.3f\r", -(argin / um2PsConvFactor));
	write_read(instr,NULL,false,0);
	/*----- PROTECTED REGION END -----*/	//	PZ116E::backward
}
//--------------------------------------------------------
/**
 *	Command Stop related method
 *	Description: Stop motion
 *
 */
//--------------------------------------------------------
void PZ116E::stop()
{
	DEBUG_STREAM << "PZ116E::Stop()  - " << device_name << endl;
	/*----- PROTECTED REGION ID(PZ116E::stop) ENABLED START -----*/

	//	Add your own code
	char instr[2] = {24, '\r'};
	write_read(instr,NULL,false,0);

	/*----- PROTECTED REGION END -----*/	//	PZ116E::stop
}
//--------------------------------------------------------
/**
 *	Command Reset related method
 *	Description: Reset controller
 *
 */
//--------------------------------------------------------
void PZ116E::reset()
{
	DEBUG_STREAM << "PZ116E::Reset()  - " << device_name << endl;
	/*----- PROTECTED REGION ID(PZ116E::reset) ENABLED START -----*/

	//	Add your own code
	/* clean errors */
	try {
		check_err();
	}
	catch(Tango::DevFailed e) {}

	write_read((char *)"RST\r",NULL,false,0);
	clock_gettime(CLOCK_MONOTONIC, &last_reset_time);
	attr_Stat_read[PZ116E_STATE_RESET] = true;

	/*----- PROTECTED REGION END -----*/	//	PZ116E::reset
}
//--------------------------------------------------------
/**
 *	Command StartWaveform related method
 *	Description: 
 *
 */
//--------------------------------------------------------
void PZ116E::start_waveform()
{
	DEBUG_STREAM << "PZ116E::StartWaveform()  - " << device_name << endl;
	/*----- PROTECTED REGION ID(PZ116E::start_waveform) ENABLED START -----*/

	//	Add your own code
	char instr[PZ116E_STRING_SIZE], outstr[PZ116E_STRING_SIZE];

	try {
		write_read((char *)"WTO A 0\r",NULL,false,0);
		/* go to start position */
		sprintf(instr,"MOV A %.3f\r", attr_Waveform_read[0]/um2PsConvFactor);
		write_read(instr, NULL, false,0);
		/* check errors */
		check_err();
		/* wait until arrived on target */
		int count = 0;
		while (count < 20) {
			write_read((char *)"ONT? A\r",outstr,true,0);
			if (outstr[0] == '1') break;
			count++; usleep(50000);
		}
		if (count >= 20) {
			TangoSys_MemStream out_stream;
  		out_stream << "Can't reach scan start value " << attr_Waveform_read[0] << ends;
			Tango::Except::throw_exception(
				out_stream.str(), (const char *) "Failed to start scan",
				(const char *) "Z116E::start_waveform()", Tango::ERR);
		}
		write_read((char *)"SPA A 12 0 \r",NULL,false,0);
		/* start waveform on trigger */
		if (*attr_OneshotWaveform_read)
			sprintf(instr,"WTO A 255 0\r");
		else
			sprintf(instr,"WTO A %d 0\r", (int)wsize);
		write_read(instr,NULL,false,0);
		/* check errors */
		check_err();
	}
	catch(Tango::DevFailed &e) {
		Tango::Except::re_throw_exception(e,
			(const char *)"Error from serial line",
			(const char *)"Can't start scan",
			(const char *) "PZ116E::start_waveform()", Tango::ERR);
	}	
	/* disable realtime mode during trigger scan */
	backup_rt_mode = *attr_RtMode_read;
	*attr_RtMode_read = false;
	attr_Stat_read[PZ116E_STATE_SCAN_START] = true;

	/*----- PROTECTED REGION END -----*/	//	PZ116E::start_waveform
}
//--------------------------------------------------------
/**
 *	Command StopWaveform related method
 *	Description: 
 *
 */
//--------------------------------------------------------
void PZ116E::stop_waveform()
{
	DEBUG_STREAM << "PZ116E::StopWaveform()  - " << device_name << endl;
	/*----- PROTECTED REGION ID(PZ116E::stop_waveform) ENABLED START -----*/

	//	Add your own code
	/* move on trigger */
	write_read((char *)"WTO A 0\r",NULL,false,0);
	
	if (attr_Stat_read[PZ116E_STATE_SCAN_START] == true) {
		attr_Stat_read[PZ116E_STATE_SCAN_START] = false;
		*attr_RtMode_read = backup_rt_mode ;
		start_wave_idx = 0;
		stop_wave_idx = 0;
		if (*attr_RtMode_read) {
			*attr_Position_read = attr_Waveform_read[wsize-1];
			attr_position->set_write_value(attr_Waveform_read[wsize-1]);
		}
	}

	/*----- PROTECTED REGION END -----*/	//	PZ116E::stop_waveform
}
//--------------------------------------------------------
/**
 *	Command SendCmd related method
 *	Description: 
 *
 *	@param argin 
 *	@returns 
 */
//--------------------------------------------------------
Tango::DevString PZ116E::send_cmd(Tango::DevString argin)
{
	Tango::DevString argout;
	DEBUG_STREAM << "PZ116E::SendCmd()  - " << device_name << endl;
	/*----- PROTECTED REGION ID(PZ116E::send_cmd) ENABLED START -----*/

	//	Add your own code
	char strin[1000], strout[1000];
	bool readback = false;	
	if (strlen((char*) argin) >= 999) {
		Tango::Except::throw_exception(
			(const char *) "Failed to send command",
			(const char *) "Too large string",
 			(const char *) "PZ116E::send_cmd", Tango::ERR);		
	}
	memset(strout, 0, 1000);
	sprintf(strin,"%s\r",(char *)argin);
	for (unsigned int i = 0; i < strlen(strin); i++) {
		if (strin[i] == '?')
			readback = true;
	}
	write_read(strin, strout,readback, 10000);

	argout = CORBA::string_dup(strout);	
	
	/*----- PROTECTED REGION END -----*/	//	PZ116E::send_cmd
	return argout;
}

/*----- PROTECTED REGION ID(PZ116E::namespace_ending) ENABLED START -----*/

	//	Additional Methods

//+------------------------------------------------------------------
/**
 *	method:	PZ116E::write_read
 *
 *	description:	method to write read from twt ampl
 *
 */
//+------------------------------------------------------------------
void PZ116E::write_read(char *instr, char *outstr, bool rx, long plus_usleep)
{
	DEBUG_STREAM << "PZ116E::write_read(): entering... !" << endl;

	//	Add your own code to control device here
	int in_len = strlen(instr);
	
	INFO_STREAM<< "instr: " << instr << endl;

	if (instr[in_len-1] != 13)  {
		instr[in_len] = 13; /* add CR(13) */
	}

	mutex_cmd->lock();

	if (write(tty, instr, in_len) != in_len) {
		INFO_STREAM << "write_read(): write error, " << instr << endl;
		attr_Stat_read[PZ116E_STATE_COMM_ERR] = true;
		mutex_cmd->unlock();
		Tango::Except::throw_exception(
			(const char *) "Serial communication error",
			(const char *) "Error writing to serial line",
 			(const char *) "PZ116E::write_read", Tango::ERR);		
	
	}

	usleep(txRxSleep + plus_usleep); /* sleep between tx and rx, sleep also from txrx handshakes */

	if (!rx) {
		mutex_cmd->unlock();
		return;
	}

	memset(outstr, 0, PZ116E_STRING_SIZE);

	if (read(tty, outstr, PZ116E_STRING_SIZE) == 0) {
		INFO_STREAM << "Error (rx): no data from serial line " << serialLine << endl;
		attr_Stat_read[PZ116E_STATE_COMM_ERR] = true;
		mutex_cmd->unlock();
		Tango::Except::throw_exception(
			(const char *) "Serial communication error",
			(const char *) "No data from serial line",
			(const char *) "PZ116E::write_read", Tango::ERR);
	}

	/* look for LF(10) char */
	long found_flag = 0;
	for (unsigned int i = 0; i < strlen(outstr); i++) {
		if (outstr[i] == 10) {
			outstr[i] = 0;
			found_flag++;
		}
	}

	/* it must be only one LF */
	if (found_flag != 1) {
		INFO_STREAM << "Error (rx): corrupted string: " << outstr << endl;
		attr_Stat_read[PZ116E_STATE_COMM_ERR] = true;
		mutex_cmd->unlock();
		Tango::Except::throw_exception(
			(const char *) "Serial communication error",
			(const char *) "Corrupted string from serial line",
			(const char *) "PZ116E::write_read", Tango::ERR);
	}

	attr_Stat_read[PZ116E_STATE_COMM_ERR] = false;
	mutex_cmd->unlock();

	INFO_STREAM << "outstr: " << outstr << endl;

}


//+------------------------------------------------------------------
/**
 *	method:	PZ116E::open_tty
 *
 *	description:	
 *
 */
//+------------------------------------------------------------------
bool PZ116E::open_tty(void)
{
	tty = 0;

	if ((tty = open(serialLine.c_str(), O_RDWR | O_NOCTTY | O_NONBLOCK)) < 0) {
		printf("PZ116E::open_tty: error opening %s\n", serialLine.c_str());		
		return false;
	}
	else {
		printf("%s opened successfully (fd=%d)\n",serialLine.c_str(),tty);
	}
		
	/* save current modem settings */		
	tcgetattr(tty, &attr_tty);
			
	attr_tty.c_cflag = B115200 | CS8 | CLOCAL | CREAD;
 
	/* Ignore bytes with parity errors and make terminal raw and dumb */
	attr_tty.c_iflag = IGNPAR;
 
	/* Raw output. */
	attr_tty.c_oflag = 0;
 
	/* Don't echo characters and don't generate signals. */
	attr_tty.c_lflag = 0;	
	attr_tty.c_cc[VMIN] = 0;	
	attr_tty.c_cc[VTIME] = 0;	/* minimum time to read chars after their arrival */

	/* now clean the modem line and activate the settings for modem */
 	tcflush(tty, TCIFLUSH);
	tcsetattr(tty, TCSANOW, &attr_tty);	

	return true;

}


//+------------------------------------------------------------------
/**
 *	method:	PZ116E::close_tty
 *
 *	description:	
 *
 */
//+------------------------------------------------------------------
void PZ116E::close_tty(void)
{
	if (tty != 0)
		close(tty);

}


//+------------------------------------------------------------------
/**
 *	method:	PZ116E::check_err
 *
 *	description:	method to check cmd error
 *
 */
//+------------------------------------------------------------------
void PZ116E::check_err()
{
	char outstr[PZ116E_STRING_SIZE];
	write_read((char *)"ERR?\r",outstr,true,0);

	if (outstr[0] == '0')
		return;

	for (int i = 0; i < PZ116E_NUM_ERRORS/2 ; i++) {
			if ((strcmp(err_msg_table[i*2], outstr) == 0)) {
				TangoSys_MemStream out_stream;
				out_stream << err_msg_table[i*2+1] << ends;
				Tango::Except::throw_exception(
					out_stream.str(),
					(const char *) "Command failed",
					(const char *) "PZ116E::check_err", Tango::ERR);
		}
	}

}


//+------------------------------------------------------------------
/**
 *	method:	PZ116E::init_pi
 *
 *	description:	method to check cmd error
 *
 */
//+------------------------------------------------------------------
void PZ116E::init_pi()
{
	char instr[PZ116E_STRING_SIZE], outstr[PZ116E_STRING_SIZE];

	/* disable trigger mode  */
	write_read((char *)"WTO A 0\r",outstr,false,0);
	/* enable close loop  */
	if (closedLoop) {
		write_read((char *)"SVO A 1\r",outstr,false,0);
	}
	else {
		write_read((char *)"SVO A 0\r",outstr,false,0);
	}
	/* disable move on trigger */
	write_read((char *)"MVT A 0\r",outstr,false,0);
	/* get identity number */
	write_read((char *)"*IDN?\r",idn_name,true,0);

	bool wsize_found = false;
	for (int i = 0; i < 256; i++) {
		sprintf(instr,"SWT? A %d\r",255-i);
		write_read(instr ,outstr, true, 10000);
		attr_Waveform_read[255-i] = (double)atof(outstr)*um2PsConvFactor;
		if ((attr_Waveform_read[255] != attr_Waveform_read[255-i]) && (wsize_found == false)) {
			wsize_found = true;
			wsize = 255 - i + 2;
		}
	}
	

	check_err();

}	


//+------------------------------------------------------------------
/**
 *	method:	PZ116E::set_DevState
 *
 *	description:	method to check cmd error
 *
 */
//+------------------------------------------------------------------
void PZ116E::set_DevState()
{

	int exp_time;

	std::stringstream s;

	if (*attr_RtMode_read) {
		s << "RT mode enabled" << endl;
		set_status(s.str());
		return;
	}

	if (attr_Stat_read[PZ116E_STATE_SCAN_START])
		exp_time = 250000;
	else
		exp_time = 500000;

	struct timespec now;
	clock_gettime(CLOCK_MONOTONIC, &now);

	if (abs((now.tv_sec - last_state_time.tv_sec)*1000000+(now.tv_nsec - 		last_state_time.tv_nsec)/1000) < exp_time)
		return;

	last_state_time.tv_sec = now.tv_sec;	
	
	/* get position */
	try {
		char outstr[PZ116E_STRING_SIZE];
		/* get real position */
		if (closedLoop) {
			write_read((char *)"POS? A\r",outstr,true,0);
		}
		else {
			write_read((char *)"SVA? A\r",outstr,true,0);
		}
		*attr_Position_read = um2PsConvFactor * (double)atof(outstr);
		/* get readback setting */

		if (closedLoop)
			write_read((char *)"MOV? A\r",outstr,true,0);
		else
			write_read((char *)"SVA? A\r",outstr,true,0);
		attr_position->set_write_value((double)atof(outstr));
		/* get on target status */
		write_read((char *)"ONT? A\r",outstr,true,0);
		if ((outstr[0] == '0') && closedLoop) {
			set_state(Tango::MOVING);
			s << "Piezo MOVING" << endl;
		}
		else {
			set_state(Tango::ON);
			s << "Piezo ON target" << endl;
		}

		/* get running waveform on trigger status */
		write_read((char *)"WTO? A\r",outstr,true,0);
		if (outstr[0] != '0') {
			set_state(Tango::RUNNING);
			s << "Waveform RUNNING" << endl;
		}

		if (attr_Stat_read[PZ116E_STATE_RESET]) {
			set_state(Tango::INIT); 
			s << "INIT procedure  (" << 10 - (now.tv_sec-last_reset_time.tv_sec) << "sec.)" << endl;
		}

	}
	catch(Tango::DevFailed e) {
		set_state(Tango::FAULT); 
		s << "Communication breakdown" << endl;
	}	

	/* init after a reset */
	if (((now.tv_sec-last_reset_time.tv_sec) > 10) && attr_Stat_read[PZ116E_STATE_RESET]) {
		try {
			init_pi();
			if (closedLoop) {
				write_read((char *)"MOV A 0\r",NULL,false,0);
			}
			else {
				//write_read((char *)"SVA A 0\r",NULL,false,0);
			}
			attr_Stat_read[PZ116E_STATE_RESET] = false;
		}
		catch(Tango::DevFailed e) {}
	}


	if (*attr_OneshotWaveform_read && (get_state() != (Tango::FAULT)) && (attr_Stat_read[PZ116E_STATE_SCAN_START] == true)) {
		/* if end position target differs by 1 um then disable triggered scan */
		if ((fabs(*attr_Position_read - attr_Waveform_read[wsize-1])/um2PsConvFactor) < 0.1) {
			stop_waveform();
		}
	}

	set_status(s.str());

}	

	/*----- PROTECTED REGION END -----*/	//	PZ116E::namespace_ending
} //	namespace
