/*----- PROTECTED REGION ID(PZ116E.h) ENABLED START -----*/
//=============================================================================
//
// file :        PZ116E.h
//
// description : Include for the PZ116E class.
//
// project :     PI Piezo PZ116E controller device server.
//
// $Author: giulio $
//
// $Revision: 1.3 $
// $Date: 2015-08-27 14:51:04 $
//
// SVN only:
// $HeadURL:  $
//
// CVS only:
// $Source: /home/cvsadm/cvsroot/fermi/servers/pz116e/src/PZ116E.h,v $
// $Log: PZ116E.h,v $
// Revision 1.3  2015-08-27 14:51:04  giulio
// Supported open-loop mode
//
// Revision 1.2  2012-06-07 09:57:21  giulio
// Added status strings, load waveform in init procedure
//
// Revision 1.1.1.1  2012-06-06 14:03:00  giulio
// First release
//
//
//=============================================================================
//                This file is generated by POGO
//        (Program Obviously used to Generate tango Object)
//=============================================================================


#ifndef PZ116E_H
#define PZ116E_H


#include <termios.h>
#include <stdio.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <time.h>
#include <signal.h>
#include <math.h>

#include <tango.h>

#define PZ116E_STRING_SIZE 50

#define PZ116E_NUM_STATES           4
#define PZ116E_STATE_MOVING        0
#define PZ116E_STATE_COMM_ERR   1
#define PZ116E_STATE_SCAN_START 2
#define PZ116E_STATE_RESET               3

#define PZ116E_NUM_ERRORS        42   /* 21 errors x 2 */
#define PZ116E_ERR_MSG_SIZE      200


/*----- PROTECTED REGION END -----*/	//	PZ116E.h

/**
 *  PZ116E class description:
 *    PI Piezo PZ116E controller device server
 */

namespace PZ116E_ns
{
/*----- PROTECTED REGION ID(PZ116E::Additional Class Declarations) ENABLED START -----*/

		//		Additional Class Declarations

	/*----- PROTECTED REGION END -----*/	//	PZ116E::Additional Class Declarations

class PZ116E : public TANGO_BASE_CLASS
{

/*----- PROTECTED REGION ID(PZ116E::Data Members) ENABLED START -----*/

	//		Add your own data members
	omni_mutex::omni_mutex *mutex_cmd;
  struct termios attr_tty;
	int tty;
	struct timespec last_state_time, last_reset_time, last_pos_time;
	Tango::WAttribute *attr_position;	
	long wsize;
	bool backup_rt_mode;
	char idn_name[PZ116E_STRING_SIZE];
	long start_wave_idx, stop_wave_idx;
	/*----- PROTECTED REGION END -----*/	//	PZ116E::Data Members

//	Device property data members
public:
	//	InitCommand:	
	vector<string>	initCommand;
	//	SerialLine:	Linux serial device (ex: /dev/ttyS0)
	string	serialLine;
	//	Um2PsConvFactor:	Conversion factor ps =  Um2PsConvFactor * um
	Tango::DevDouble	um2PsConvFactor;
	//	TxRxSleep:	ms to sleep between tx and rx
	Tango::DevLong	txRxSleep;
	//	ClosedLoop:	
	Tango::DevBoolean	closedLoop;

//	Attribute data members
public:
	Tango::DevBoolean	*attr_RtMode_read;
	Tango::DevDouble	*attr_Position_read;
	Tango::DevString	*attr_IdentityNumber_read;
	Tango::DevBoolean	*attr_OneshotWaveform_read;
	Tango::DevBoolean	*attr_Stat_read;
	Tango::DevDouble	*attr_Waveform_read;

//	Constructors and destructors
public:
	/**
	 * Constructs a newly device object.
	 *
	 *	@param cl	Class.
	 *	@param s 	Device Name
	 */
	PZ116E(Tango::DeviceClass *cl,string &s);
	/**
	 * Constructs a newly device object.
	 *
	 *	@param cl	Class.
	 *	@param s 	Device Name
	 */
	PZ116E(Tango::DeviceClass *cl,const char *s);
	/**
	 * Constructs a newly device object.
	 *
	 *	@param cl	Class.
	 *	@param s 	Device name
	 *	@param d	Device description.
	 */
	PZ116E(Tango::DeviceClass *cl,const char *s,const char *d);
	/**
	 * The device object destructor.
	 */	
	~PZ116E() {delete_device();};


//	Miscellaneous methods
public:
	/*
	 *	will be called at device destruction or at init command.
	 */
	void delete_device();
	/*
	 *	Initialize the device
	 */
	virtual void init_device();
	/*
	 *	Read the device properties from database
	 */
	void get_device_property();
	/*
	 *	Always executed method before execution command method.
	 */
	virtual void always_executed_hook();


//	Attribute methods
public:
	//--------------------------------------------------------
	/*
	 *	Method      : PZ116E::read_attr_hardware()
	 *	Description : Hardware acquisition for attributes.
	 */
	//--------------------------------------------------------
	virtual void read_attr_hardware(vector<long> &attr_list);
	//--------------------------------------------------------
	/*
	 *	Method      : PZ116E::write_attr_hardware()
	 *	Description : Hardware writing for attributes.
	 */
	//--------------------------------------------------------
	virtual void write_attr_hardware(vector<long> &attr_list);

/**
 *	Attribute RtMode related methods
 *	Description: If enabled, do not readback position, do not check errors in mov command
 *
 *	Data type:	Tango::DevBoolean
 *	Attr type:	Scalar
 */
	virtual void read_RtMode(Tango::Attribute &attr);
	virtual void write_RtMode(Tango::WAttribute &attr);
	virtual bool is_RtMode_allowed(Tango::AttReqType type);
/**
 *	Attribute Position related methods
 *	Description: 
 *
 *	Data type:	Tango::DevDouble
 *	Attr type:	Scalar
 */
	virtual void read_Position(Tango::Attribute &attr);
	virtual void write_Position(Tango::WAttribute &attr);
	virtual bool is_Position_allowed(Tango::AttReqType type);
/**
 *	Attribute IdentityNumber related methods
 *	Description: 
 *
 *	Data type:	Tango::DevString
 *	Attr type:	Scalar
 */
	virtual void read_IdentityNumber(Tango::Attribute &attr);
	virtual bool is_IdentityNumber_allowed(Tango::AttReqType type);
/**
 *	Attribute OneshotWaveform related methods
 *	Description: 
 *
 *	Data type:	Tango::DevBoolean
 *	Attr type:	Scalar
 */
	virtual void read_OneshotWaveform(Tango::Attribute &attr);
	virtual void write_OneshotWaveform(Tango::WAttribute &attr);
	virtual bool is_OneshotWaveform_allowed(Tango::AttReqType type);
/**
 *	Attribute Stat related methods
 *	Description: 0) moving=true. not moving=false
 *               1) comm. error=true comm. ok=false
 *               
 *
 *	Data type:	Tango::DevBoolean
 *	Attr type:	Spectrum max = 32
 */
	virtual void read_Stat(Tango::Attribute &attr);
	virtual bool is_Stat_allowed(Tango::AttReqType type);
/**
 *	Attribute Waveform related methods
 *	Description: 
 *
 *	Data type:	Tango::DevDouble
 *	Attr type:	Spectrum max = 256
 */
	virtual void read_Waveform(Tango::Attribute &attr);
	virtual void write_Waveform(Tango::WAttribute &attr);
	virtual bool is_Waveform_allowed(Tango::AttReqType type);


	//--------------------------------------------------------
	/**
	 *	Method      : PZ116E::add_dynamic_attributes()
	 *	Description : Add dynamic attributes if any.
	 */
	//--------------------------------------------------------
	void add_dynamic_attributes();



//	Command related methods
public:
	/**
	 *	Command Forward related method
	 *	Description: Move forward
	 *
	 *	@param argin 
	 */
	virtual void forward(Tango::DevDouble argin);
	virtual bool is_Forward_allowed(const CORBA::Any &any);
	/**
	 *	Command Backward related method
	 *	Description: Move backward
	 *
	 *	@param argin 
	 */
	virtual void backward(Tango::DevDouble argin);
	virtual bool is_Backward_allowed(const CORBA::Any &any);
	/**
	 *	Command Stop related method
	 *	Description: Stop motion
	 *
	 */
	virtual void stop();
	virtual bool is_Stop_allowed(const CORBA::Any &any);
	/**
	 *	Command Reset related method
	 *	Description: Reset controller
	 *
	 */
	virtual void reset();
	virtual bool is_Reset_allowed(const CORBA::Any &any);
	/**
	 *	Command StartWaveform related method
	 *	Description: 
	 *
	 */
	virtual void start_waveform();
	virtual bool is_StartWaveform_allowed(const CORBA::Any &any);
	/**
	 *	Command StopWaveform related method
	 *	Description: 
	 *
	 */
	virtual void stop_waveform();
	virtual bool is_StopWaveform_allowed(const CORBA::Any &any);
	/**
	 *	Command SendCmd related method
	 *	Description: 
	 *
	 *	@param argin 
	 *	@returns 
	 */
	virtual Tango::DevString send_cmd(Tango::DevString argin);
	virtual bool is_SendCmd_allowed(const CORBA::Any &any);


/*----- PROTECTED REGION ID(PZ116E::Additional Method prototypes) ENABLED START -----*/

	//	Additional Method prototypes
	void write_read(char *instr, char *outstr, bool norx, long plus_usleep);
	bool open_tty(void);
	void close_tty(void);
	void check_err(void);
	void init_pi(void);
	void set_DevState(void);
	/*----- PROTECTED REGION END -----*/	//	PZ116E::Additional Method prototypes
};

/*----- PROTECTED REGION ID(PZ116E::Additional Classes Definitions) ENABLED START -----*/

	//	Additional Classes definitions

	/*----- PROTECTED REGION END -----*/	//	PZ116E::Additional Classes Definitions

}	//	End of namespace

#endif   //	PZ116E_H
